@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2021 Apple Inc. All rights reserved.
+ * Copyright (C) 2021-2023 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -31,43 +31,56 @@
 #include "RemoteAdapterMessages.h"
 #include "RemoteDeviceProxy.h"
 #include "WebGPUConvertToBackingContext.h"
+#include <wtf/TZoneMallocInlines.h>
 
 namespace WebKit::WebGPU {
 
-RemoteAdapterProxy::RemoteAdapterProxy(String&& name, PAL::WebGPU::SupportedFeatures& features, PAL::WebGPU::SupportedLimits& limits, bool isFallbackAdapter, RemoteGPUProxy& parent, ConvertToBackingContext& convertToBackingContext, WebGPUIdentifier identifier)
+WTF_MAKE_TZONE_ALLOCATED_IMPL(RemoteAdapterProxy);
+
+RemoteAdapterProxy::RemoteAdapterProxy(String&& name, WebCore::WebGPU::SupportedFeatures& features, WebCore::WebGPU::SupportedLimits& limits, bool isFallbackAdapter, bool xrCompatible, RemoteGPUProxy& parent, ConvertToBackingContext& convertToBackingContext, WebGPUIdentifier identifier)
     : Adapter(WTFMove(name), features, limits, isFallbackAdapter)
     , m_backing(identifier)
     , m_convertToBackingContext(convertToBackingContext)
     , m_parent(parent)
+    , m_xrCompatible(xrCompatible)
 {
 }
 
 RemoteAdapterProxy::~RemoteAdapterProxy()
 {
+    auto sendResult = send(Messages::RemoteAdapter::Destruct());
+    UNUSED_VARIABLE(sendResult);
 }
 
-void RemoteAdapterProxy::requestDevice(const PAL::WebGPU::DeviceDescriptor& descriptor, CompletionHandler<void(Ref<PAL::WebGPU::Device>&&)>&& callback)
+void RemoteAdapterProxy::requestDevice(const WebCore::WebGPU::DeviceDescriptor& descriptor, CompletionHandler<void(RefPtr<WebCore::WebGPU::Device>&&)>&& callback)
 {
-    auto convertedDescriptor = m_convertToBackingContext->convertToBacking(descriptor);
+    Ref convertToBackingContext = m_convertToBackingContext;
+    auto convertedDescriptor = convertToBackingContext->convertToBacking(descriptor);
     ASSERT(convertedDescriptor);
     if (!convertedDescriptor)
-        return;
+        return callback(nullptr);
 
     auto identifier = WebGPUIdentifier::generate();
     auto queueIdentifier = WebGPUIdentifier::generate();
-    SupportedFeatures supportedFeatures;
-    SupportedLimits supportedLimits;
-    auto sendResult = sendSync(Messages::RemoteAdapter::RequestDevice(*convertedDescriptor, identifier, queueIdentifier), { supportedFeatures, supportedLimits });
-    if (!sendResult)
-        return;
+    auto sendResult = sendSync(Messages::RemoteAdapter::RequestDevice(*convertedDescriptor, identifier, queueIdentifier));
+    if (!sendResult.succeeded())
+        return callback(nullptr);
 
-    auto resultSupportedFeatures = PAL::WebGPU::SupportedFeatures::create(WTFMove(supportedFeatures.features));
-    auto resultSupportedLimits = PAL::WebGPU::SupportedLimits::create(
+    auto [supportedFeatures, supportedLimits] = sendResult.takeReply();
+    if (!supportedLimits.maxTextureDimension2D) {
+        callback(nullptr);
+        return;
+    }
+
+    auto resultSupportedFeatures = WebCore::WebGPU::SupportedFeatures::create(WTFMove(supportedFeatures.features));
+    auto resultSupportedLimits = WebCore::WebGPU::SupportedLimits::create(
         supportedLimits.maxTextureDimension1D,
         supportedLimits.maxTextureDimension2D,
         supportedLimits.maxTextureDimension3D,
         supportedLimits.maxTextureArrayLayers,
         supportedLimits.maxBindGroups,
+        supportedLimits.maxBindGroupsPlusVertexBuffers,
+        supportedLimits.maxBindingsPerBindGroup,
         supportedLimits.maxDynamicUniformBuffersPerPipelineLayout,
         supportedLimits.maxDynamicStorageBuffersPerPipelineLayout,
         supportedLimits.maxSampledTexturesPerShaderStage,
@@ -80,17 +93,30 @@ void RemoteAdapterProxy::requestDevice(const PAL::WebGPU::DeviceDescriptor& desc
         supportedLimits.minUniformBufferOffsetAlignment,
         supportedLimits.minStorageBufferOffsetAlignment,
         supportedLimits.maxVertexBuffers,
+        supportedLimits.maxBufferSize,
         supportedLimits.maxVertexAttributes,
         supportedLimits.maxVertexBufferArrayStride,
         supportedLimits.maxInterStageShaderComponents,
+        supportedLimits.maxInterStageShaderVariables,
+        supportedLimits.maxColorAttachments,
+        supportedLimits.maxColorAttachmentBytesPerSample,
         supportedLimits.maxComputeWorkgroupStorageSize,
         supportedLimits.maxComputeInvocationsPerWorkgroup,
         supportedLimits.maxComputeWorkgroupSizeX,
         supportedLimits.maxComputeWorkgroupSizeY,
         supportedLimits.maxComputeWorkgroupSizeZ,
-        supportedLimits.maxComputeWorkgroupsPerDimension
+        supportedLimits.maxComputeWorkgroupsPerDimension,
+        supportedLimits.maxStorageBuffersInFragmentStage,
+        supportedLimits.maxStorageTexturesInFragmentStage
     );
-    callback(RemoteDeviceProxy::create(WTFMove(resultSupportedFeatures), WTFMove(resultSupportedLimits), *this, m_convertToBackingContext, identifier, queueIdentifier));
+    auto result = RemoteDeviceProxy::create(WTFMove(resultSupportedFeatures), WTFMove(resultSupportedLimits), *this, convertToBackingContext, identifier, queueIdentifier);
+    result->setLabel(WTFMove(convertedDescriptor->label));
+    callback(WTFMove(result));
+}
+
+bool RemoteAdapterProxy::xrCompatible()
+{
+    return m_xrCompatible;
 }
 
 } // namespace WebKit::WebGPU
